# Generated by roxygen2: do not edit by hand

S3method(destructure,data.frame)
S3method(destructure,default)
S3method(destructure,summary.lm)
export("%->%")
export("%<-%")
export(destructure)
export(zeallous)
